var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
import { Fragment } from 'preact';
import { useMemo, useRef, useState } from 'preact/hooks';
import { SurveyQuestionType, } from '../../../posthog-surveys-types';
import { isArray, isNull, isNumber, isString } from '../../../utils/type-utils';
import { checkSVG, dissatisfiedEmoji, neutralEmoji, satisfiedEmoji, veryDissatisfiedEmoji, verySatisfiedEmoji, } from '../icons';
import { getDisplayOrderChoices, useSurveyContext } from '../surveys-extension-utils';
import { BottomSection } from './BottomSection';
import { QuestionHeader } from './QuestionHeader';
export function OpenTextQuestion(_a) {
    var question = _a.question, forceDisableHtml = _a.forceDisableHtml, appearance = _a.appearance, onSubmit = _a.onSubmit, onPreviewSubmit = _a.onPreviewSubmit, initialValue = _a.initialValue;
    var _b = __read(useState(function () {
        if (isString(initialValue)) {
            return initialValue;
        }
        return '';
    }), 2), text = _b[0], setText = _b[1];
    return (<Fragment>
            <div className="question-container">
                <QuestionHeader question={question.question} description={question.description} descriptionContentType={question.descriptionContentType} forceDisableHtml={forceDisableHtml}/>
                <textarea rows={4} placeholder={appearance === null || appearance === void 0 ? void 0 : appearance.placeholder} onInput={function (e) {
            setText(e.currentTarget.value);
            e.stopPropagation();
        }} onKeyDown={function (e) {
            e.stopPropagation();
        }} value={text}/>
            </div>
            <BottomSection text={question.buttonText || 'Submit'} submitDisabled={!text && !question.optional} appearance={appearance} onSubmit={function () { return onSubmit(text); }} onPreviewSubmit={function () { return onPreviewSubmit(text); }}/>
        </Fragment>);
}
export function LinkQuestion(_a) {
    var question = _a.question, forceDisableHtml = _a.forceDisableHtml, appearance = _a.appearance, onSubmit = _a.onSubmit, onPreviewSubmit = _a.onPreviewSubmit;
    return (<Fragment>
            <div className="question-container">
                <QuestionHeader question={question.question} description={question.description} descriptionContentType={question.descriptionContentType} forceDisableHtml={forceDisableHtml}/>
            </div>
            <BottomSection text={question.buttonText || 'Submit'} submitDisabled={false} link={question.link} appearance={appearance} onSubmit={function () { return onSubmit('link clicked'); }} onPreviewSubmit={function () { return onPreviewSubmit('link clicked'); }}/>
        </Fragment>);
}
export function RatingQuestion(_a) {
    var question = _a.question, forceDisableHtml = _a.forceDisableHtml, displayQuestionIndex = _a.displayQuestionIndex, appearance = _a.appearance, onSubmit = _a.onSubmit, onPreviewSubmit = _a.onPreviewSubmit, initialValue = _a.initialValue;
    var scale = question.scale;
    var starting = question.scale === 10 ? 0 : 1;
    var _b = __read(useState(function () {
        if (isNumber(initialValue)) {
            return initialValue;
        }
        if (isArray(initialValue) && initialValue.length > 0 && isNumber(parseInt(initialValue[0]))) {
            return parseInt(initialValue[0]);
        }
        if (isString(initialValue) && isNumber(parseInt(initialValue))) {
            return parseInt(initialValue);
        }
        return null;
    }), 2), rating = _b[0], setRating = _b[1];
    var isPreviewMode = useSurveyContext().isPreviewMode;
    var handleSubmit = function (num) {
        if (isPreviewMode) {
            return onPreviewSubmit(num);
        }
        return onSubmit(num);
    };
    return (<Fragment>
            <div className="question-container">
                <QuestionHeader question={question.question} description={question.description} descriptionContentType={question.descriptionContentType} forceDisableHtml={forceDisableHtml}/>
                <div className="rating-section">
                    <div className="rating-options">
                        {question.display === 'emoji' && (<div className="rating-options-emoji">
                                {(question.scale === 3 ? threeScaleEmojis : fiveScaleEmojis).map(function (emoji, idx) {
                var active = idx + 1 === rating;
                return (<button aria-label={"Rate ".concat(idx + 1)} className={"ratings-emoji question-".concat(displayQuestionIndex, "-rating-").concat(idx, " ").concat(active ? 'rating-active' : '')} value={idx + 1} key={idx} type="button" onClick={function () {
                        var response = idx + 1;
                        setRating(response);
                        if (question.skipSubmitButton) {
                            handleSubmit(response);
                        }
                    }}>
                                            {emoji}
                                        </button>);
            })}
                            </div>)}
                        {question.display === 'number' && (<div className="rating-options-number" style={{ gridTemplateColumns: "repeat(".concat(scale - starting + 1, ", minmax(0, 1fr))") }}>
                                {getScaleNumbers(question.scale).map(function (number, idx) {
                var active = rating === number;
                return (<RatingButton key={idx} displayQuestionIndex={displayQuestionIndex} active={active} appearance={appearance} num={number} setActiveNumber={function (response) {
                        setRating(response);
                        if (question.skipSubmitButton) {
                            handleSubmit(response);
                        }
                    }}/>);
            })}
                            </div>)}
                    </div>
                    <div className="rating-text">
                        <div>{question.lowerBoundLabel}</div>
                        <div>{question.upperBoundLabel}</div>
                    </div>
                </div>
            </div>
            <BottomSection text={question.buttonText || (appearance === null || appearance === void 0 ? void 0 : appearance.submitButtonText) || 'Submit'} submitDisabled={isNull(rating) && !question.optional} appearance={appearance} onSubmit={function () { return onSubmit(rating); }} onPreviewSubmit={function () { return onPreviewSubmit(rating); }} skipSubmitButton={question.skipSubmitButton}/>
        </Fragment>);
}
export function RatingButton(_a) {
    var num = _a.num, active = _a.active, displayQuestionIndex = _a.displayQuestionIndex, setActiveNumber = _a.setActiveNumber;
    return (<button aria-label={"Rate ".concat(num)} className={"ratings-number question-".concat(displayQuestionIndex, "-rating-").concat(num, " ").concat(active ? 'rating-active' : '')} type="button" onClick={function () {
            setActiveNumber(num);
        }}>
            {num}
        </button>);
}
function isSubmitDisabled(selectedChoices, openChoiceSelected, openEndedInput, optional) {
    if (optional) {
        return false;
    }
    if (isNull(selectedChoices)) {
        return true;
    }
    if (isArray(selectedChoices)) {
        if (!openChoiceSelected && selectedChoices.length === 0) {
            return true;
        }
        if (openChoiceSelected && !openEndedInput && selectedChoices.length === 0) {
            return true;
        }
    }
    return false;
}
export function MultipleChoiceQuestion(_a) {
    var question = _a.question, forceDisableHtml = _a.forceDisableHtml, displayQuestionIndex = _a.displayQuestionIndex, appearance = _a.appearance, onSubmit = _a.onSubmit, onPreviewSubmit = _a.onPreviewSubmit, initialValue = _a.initialValue;
    var openChoiceInputRef = useRef(null);
    var choices = useMemo(function () { return getDisplayOrderChoices(question); }, [question]);
    var _b = __read(useState(function () {
        if (isString(initialValue)) {
            return initialValue;
        }
        if (isArray(initialValue)) {
            return initialValue;
        }
        return question.type === SurveyQuestionType.SingleChoice ? null : [];
    }), 2), selectedChoices = _b[0], setSelectedChoices = _b[1];
    var _c = __read(useState(function () {
        if (isString(initialValue)) {
            return !choices.includes(initialValue);
        }
        if (isArray(initialValue)) {
            // check if initialValue IS NOT in choices
            return !choices.some(function (choice) { return initialValue.includes(choice); });
        }
        return false;
    }), 2), openChoiceSelected = _c[0], setOpenChoiceSelected = _c[1];
    var _d = __read(useState(function () {
        if (isString(initialValue) && !choices.includes(initialValue)) {
            return initialValue;
        }
        if (isArray(initialValue)) {
            return initialValue.find(function (choice) { return !choices.includes(choice); }) || '';
        }
        return '';
    }), 2), openEndedInput = _d[0], setOpenEndedInput = _d[1];
    var isPreviewMode = useSurveyContext().isPreviewMode;
    var inputType = question.type === SurveyQuestionType.SingleChoice ? 'radio' : 'checkbox';
    var shouldSkipSubmit = question.skipSubmitButton && question.type === SurveyQuestionType.SingleChoice && !question.hasOpenChoice;
    var handleChoiceChange = function (val, isOpenChoice) {
        if (isOpenChoice) {
            setOpenChoiceSelected(!openChoiceSelected);
            // Focus the input when open choice is selected
            if (!openChoiceSelected) {
                // Use a small delay to ensure the focus happens after the state update
                setTimeout(function () { var _a; return (_a = openChoiceInputRef.current) === null || _a === void 0 ? void 0 : _a.focus(); }, 0);
            }
            return;
        }
        if (question.type === SurveyQuestionType.SingleChoice) {
            setSelectedChoices(val);
            setOpenChoiceSelected(false); // Deselect open choice when selecting another option
            if (shouldSkipSubmit) {
                onSubmit(val);
                if (isPreviewMode) {
                    onPreviewSubmit(val);
                }
            }
            return;
        }
        if (question.type === SurveyQuestionType.MultipleChoice && isArray(selectedChoices)) {
            if (selectedChoices.includes(val)) {
                setSelectedChoices(selectedChoices.filter(function (choice) { return choice !== val; }));
            }
            else {
                setSelectedChoices(__spreadArray(__spreadArray([], __read(selectedChoices), false), [val], false));
            }
        }
    };
    var handleOpenEndedInputChange = function (e) {
        e.stopPropagation();
        setOpenEndedInput(e.currentTarget.value);
        if (question.type === SurveyQuestionType.SingleChoice) {
            setSelectedChoices(e.currentTarget.value);
        }
    };
    return (<Fragment>
            <div className="question-container">
                <QuestionHeader question={question.question} description={question.description} descriptionContentType={question.descriptionContentType} forceDisableHtml={forceDisableHtml}/>
                <div className="multiple-choice-options limit-height">
                    {choices.map(function (choice, idx) {
            var isOpenChoice = !!question.hasOpenChoice && idx === question.choices.length - 1;
            var choiceClass = "choice-option".concat(isOpenChoice ? ' choice-option-open' : '');
            var isChecked = isOpenChoice
                ? openChoiceSelected
                : question.type === SurveyQuestionType.SingleChoice
                    ? selectedChoices === choice
                    : isArray(selectedChoices) && selectedChoices.includes(choice);
            return (<div className={choiceClass} key={idx}>
                                <input type={inputType} id={"surveyQuestion".concat(displayQuestionIndex, "Choice").concat(idx)} name={"question".concat(displayQuestionIndex)} checked={isChecked} onClick={function () { return handleChoiceChange(choice, isOpenChoice); }}/>
                                <label htmlFor={"surveyQuestion".concat(displayQuestionIndex, "Choice").concat(idx)}>
                                    {isOpenChoice ? (<>
                                            <span>{choice}:</span>
                                            <input type="text" ref={openChoiceInputRef} id={"surveyQuestion".concat(displayQuestionIndex, "Choice").concat(idx, "Open")} name={"question".concat(displayQuestionIndex)} value={openEndedInput} onKeyDown={function (e) {
                        e.stopPropagation();
                    }} onInput={function (e) { return handleOpenEndedInputChange(e); }} onClick={function (e) {
                        // Ensure the checkbox/radio gets checked when clicking the input
                        if (!openChoiceSelected) {
                            handleChoiceChange(choice, true);
                        }
                        e.stopPropagation();
                    }}/>
                                        </>) : (choice)}
                                </label>
                                <span className="choice-check">{checkSVG}</span>
                            </div>);
        })}
                </div>
            </div>
            <BottomSection text={question.buttonText || 'Submit'} submitDisabled={isSubmitDisabled(selectedChoices, openChoiceSelected, openEndedInput, !!question.optional)} appearance={appearance} onSubmit={function () {
            if (openChoiceSelected && question.type === SurveyQuestionType.MultipleChoice) {
                if (isArray(selectedChoices)) {
                    onSubmit(__spreadArray(__spreadArray([], __read(selectedChoices), false), [openEndedInput], false));
                }
            }
            else {
                onSubmit(selectedChoices);
            }
        }} onPreviewSubmit={function () {
            if (openChoiceSelected && question.type === SurveyQuestionType.MultipleChoice) {
                if (isArray(selectedChoices)) {
                    onPreviewSubmit(__spreadArray(__spreadArray([], __read(selectedChoices), false), [openEndedInput], false));
                }
            }
            else {
                onPreviewSubmit(selectedChoices);
            }
        }} skipSubmitButton={shouldSkipSubmit}/>
        </Fragment>);
}
var threeScaleEmojis = [dissatisfiedEmoji, neutralEmoji, satisfiedEmoji];
var fiveScaleEmojis = [veryDissatisfiedEmoji, dissatisfiedEmoji, neutralEmoji, satisfiedEmoji, verySatisfiedEmoji];
var fiveScaleNumbers = [1, 2, 3, 4, 5];
var sevenScaleNumbers = [1, 2, 3, 4, 5, 6, 7];
var tenScaleNumbers = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10];
function getScaleNumbers(scale) {
    switch (scale) {
        case 5:
            return fiveScaleNumbers;
        case 7:
            return sevenScaleNumbers;
        case 10:
            return tenScaleNumbers;
        default:
            return fiveScaleNumbers;
    }
}
//# sourceMappingURL=QuestionTypes.jsx.map